@@ -179,10 +179,27 @@ soup_message_io_finished (SoupMessage *msg)
 
 static void io_read (SoupSocket *sock, SoupMessage *msg);
 
+static gboolean
+request_is_idempotent (SoupMessage *msg)
+{
+	/* FIXME */
+	return (msg->method == SOUP_METHOD_GET);
+}
+
 static void
 io_error (SoupSocket *sock, SoupMessage *msg, GError *error)
 {
-	if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
+
+	if (io->mode == SOUP_MESSAGE_IO_CLIENT &&
+	    io->read_state <= SOUP_MESSAGE_IO_STATE_HEADERS &&
+	    io->read_meta_buf->len == 0 &&
+	    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT) &&
+	    request_is_idempotent (msg)) {
+		/* Connection got closed, but we can safely try again */
+		priv->io_status = SOUP_MESSAGE_IO_STATUS_QUEUED;
+	} else if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
 		if (error && error->domain == SOUP_SSL_ERROR) {
 			soup_message_set_status_full (msg,
 						      SOUP_STATUS_SSL_FAILED,
@@ -384,6 +401,7 @@ read_body_chunk (SoupMessage *msg)
 
 			if (io->need_content_sniffed) {
 				soup_message_body_append_buffer (io->sniff_data, buffer);
+				soup_buffer_free (buffer);
 				io->need_got_chunk = TRUE;
 				if (!io_handle_sniffing (msg, FALSE))
 					return FALSE;
@@ -632,7 +650,8 @@ io_write (SoupSocket *sock, SoupMessage *msg)
 				 io->write_chunk->length, TRUE))
 			return;
 
-		soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
+		if (io->mode == SOUP_MESSAGE_IO_SERVER)
+			soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
 		io->write_body_offset += io->write_chunk->length;
 		soup_buffer_free (io->write_chunk);
 		io->write_chunk = NULL;
@@ -675,7 +694,8 @@ io_write (SoupSocket *sock, SoupMessage *msg)
 				 io->write_chunk->length, TRUE))
 			return;
 
-		soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
+		if (io->mode == SOUP_MESSAGE_IO_SERVER)
+			soup_message_body_wrote_chunk (io->write_body, io->write_chunk);
 		soup_buffer_free (io->write_chunk);
 		io->write_chunk = NULL;
 
